require File.expand_path(File.dirname(__FILE__) + "/../spec_helper")
require 'haruspex/haruspex'

describe Haruspex::Haruspex do
  
  before(:each) do
    @player = Haruspex::Haruspex.new
  end

  it "should be instantiable with no paramters" do
    lambda { Haruspex::Haruspex.new }.should_not raise_error
  end
  
  it "should guess the letter which occurs in the most different words" do
    @player.word_list=["bal","bxq","bac"]

    @player.new_game(6)    
    @player.guess("___", 6).should == 'b'
  end
  
  it "should guess the next most common letter if the first guess is wrong" do
    @player.word_list=["bal","bxq","bac"]

    @player.new_game(6)
    @player.guess("___", 6).should == 'b'
    @player.guess("___", 5).should == 'a'
  end
  
  it "should partition by size" do 
    @player.word_list=["bal","bxq","bac", "aa", "ab"]

    @player.new_game(6)
    @player.guess("___", 6).should == 'b'
    @player.guess("___", 5).should == 'a'
  end
  
  it "should refresh the dictionary after every game" do
    @player.word_list=["bal","bxq","bac", "aa", "ab"]

    @player.new_game(6)
    @player.guess("___", 6).should == 'b'

    @player.new_game(6)
    @player.guess("__", 6).should == 'a'
  end
  
  it "should remove words containing incorrect letters" do
    @player.word_list=["bal","bxq","bac", "gsn", "gtd", "arb"]

    @player.new_game(6)
    @player.guess("___", 6).should == 'b'
    @player.incorrect_guess('b')
    @player.guess("___", 5).should == 'g'
  end
  
  it "should remove words not containing correct letters" do
    @player.word_list=["aaa", "bac", "btr", "tub"]

    @player.new_game(6)
    @player.guess("___", 6).should == 'b'
    @player.correct_guess('b')
    @player.guess("___", 5).should == 't'
  end
  
  it "should match patterns after a successful guess" do
    @player.word_list=["bnt", "ntk", "btr", "tub", "ltr"]

    @player.new_game(6)
    @player.guess("___", 6).should == 't'
    @player.correct_guess('t')
    @player.guess("_t_", 5).should == 'r'
  end

end